#include <amxmodx>
#include <fakemeta>
#include <hamsandwich>
#include <xs>
#include <thehidden>

#define LASERMINE_OWNER      pev_iuser2
#define LASERMINE_STEP       pev_iuser3
#define LASERMINE_HITING     pev_iuser4

#define LASERMINE_COUNT      pev_fuser1
#define LASERMINE_POWERUP    pev_fuser2
#define LASERMINE_BEAMTHINK  pev_fuser3

#define LASERMINE_BEAMENDPOINT  pev_vuser1

enum tripmine_e
{
  TRIPMINE_IDLE1 = 0,
  TRIPMINE_IDLE2,
  TRIPMINE_ARM1,
  TRIPMINE_ARM2,
  TRIPMINE_FIDGET,
  TRIPMINE_HOLSTER,
  TRIPMINE_DRAW,
  TRIPMINE_WORLD,
  TRIPMINE_GROUND,
};

enum
{
  THINK_POWERUP,
  THINK_BEAMBREAK
};

enum
{
  SOUND_POWERUP,
  SOUND_ACTIVATE,
  SOUND_STOP,
  SOUND_BEAM
};

new const g_iColors[][] = {{0, 255, 0}, {255, 0, 0}};

new const
  ENT_MODELS[]    = "models/thehidden/v_sonicmine.mdl",
  ENT_SOUND1[]    = "weapons/mine_deploy.wav",
  ENT_SOUND2[]    = "weapons/mine_charge.wav",
  ENT_SOUND3[]    = "weapons/mine_activate.wav",
  ENT_SOUND4[]    = "thehidden/sonic_beep.wav",
  ENT_SPRITE1[]   = "sprites/laserbeam.spr";

new const ENT_CLASS_NAME[] = "sonicmine";

new g_pBeamSprite, cvar_mine_count;
new g_iSettingLaser[33], g_iWasSetting[33], g_iMineCount[33], g_iLaserColor[33];
new Float:g_fPlayerSpeed[33];

public plugin_init()
{
  // Full credit to creator of lasermines SandStriker/+ARUKARI-
  register_plugin("[TH] Sonic mine", TH_VERSION, TH_AUTHOR);

  cvar_mine_count = my_register_cvar("th_mine_count", "1", "Number of sonicmines for hunters. (Default: 1)");
  register_event("CurWeapon", "Event_CurWeapon", "be", "1=1");

  register_forward(FM_Think, "Forward_Think_pre", 0);
  register_forward(FM_PlayerPostThink, "Forward_PlayerPostThink_pre", 0);

  RegisterHamPlayer(Ham_Killed, "Ham_Killed_pre", 0);
  RegisterHamPlayer(Ham_Player_ImpulseCommands, "Ham_Impulse_pre", 0);
}

public plugin_precache()
{
  precache_sound(ENT_SOUND1);
  precache_sound(ENT_SOUND2);
  precache_sound(ENT_SOUND3);
  precache_sound(ENT_SOUND4);
  precache_model(ENT_MODELS);
  g_pBeamSprite = precache_model(ENT_SPRITE1);

  return PLUGIN_CONTINUE;
}

public client_putinserver(id)
{
  remove_player_task(id);
  return PLUGIN_CONTINUE;
}

public client_disconnect(id)
{
  remove_player_task(id);
  remove_player_mines(id);
  return PLUGIN_CONTINUE;
}

public th_gamemode(mode)
{
  if(mode == GAME_ENDED || mode == GAME_RESTARTING)
  {
    new num, id;
    static players[32];
    get_players(players, num);

    for(--num; num >= 0; num--)
    {
      id = players[num];
      remove_player_task(id);
      remove_player_mines(id);
      g_iSettingLaser[id] = 0;
      g_iWasSetting[id] = 0;
      g_iLaserColor[id] = 0;
      g_iMineCount[id] = get_pcvar_num(cvar_mine_count);
    }
  }
}

public Event_CurWeapon(id)
{
  if(!g_iSettingLaser[id])
    return PLUGIN_CONTINUE;

  set_pev(id, pev_maxspeed, 1.0);
  return PLUGIN_CONTINUE;
}

public Forward_Think_pre(ent)
{
  if(!pev_valid(ent))
    return FMRES_IGNORED;

  new classname[32];
  pev(ent, pev_classname, classname, charsmax(classname));

  if(!equal(classname, ENT_CLASS_NAME))
    return FMRES_IGNORED;

  static Float:time;
  time = get_gametime();

  switch(pev(ent, LASERMINE_STEP))
  {
    case THINK_POWERUP:
    {
      new Float:powertime;
      pev(ent, LASERMINE_POWERUP, powertime);

      if(time > powertime)
      {
        set_pev(ent, pev_solid, SOLID_TRIGGER);
        set_pev(ent, LASERMINE_STEP, THINK_BEAMBREAK);

        mine_sound(ent, SOUND_ACTIVATE);
      }

      set_pev(ent, pev_nextthink, time + 0.1);
    }
    case THINK_BEAMBREAK:
    {
      static Float:vEnd[3],Float:vOrigin[3];
      pev(ent, pev_origin, vOrigin);
      pev(ent, LASERMINE_BEAMENDPOINT, vEnd);

      static id, Float:fFraction;
      engfunc(EngFunc_TraceLine, vOrigin, vEnd, DONT_IGNORE_MONSTERS, ent, 0);

      get_tr2(0, TR_flFraction, fFraction);
      id = get_tr2(0, TR_pHit);

      if(fFraction < 1.0)
      {
        if(is_user_alive(id))
        {
          set_pev(ent, pev_enemy, id);
          player_touch_laser(ent, id);
          set_pev(ent, pev_nextthink, time + random_float(0.1, 0.3));
        }
      }

      if(pev_valid(ent))
      {
        static Float:fHealth;
        pev(ent, pev_health, fHealth);

        static Float:fBeamthink;
        pev(ent, LASERMINE_BEAMTHINK, fBeamthink);

        if(fBeamthink < time)
        {
          new owner = pev(ent, LASERMINE_OWNER);
          if(pev(ent, LASERMINE_HITING) != id)
          {
            if(is_user_alive(id) && th_get_playerdata(id, PD_HIDDEN))
              g_iLaserColor[owner] = true;
            else g_iLaserColor[owner] = false;

            set_pev(ent, LASERMINE_HITING, id);
          }
          mine_laser(owner, vOrigin, vEnd);
          set_pev(ent, LASERMINE_BEAMTHINK, time + 0.1);
        }
        set_pev(ent, pev_nextthink, time + 0.01);
      }
    }
  }

  return FMRES_IGNORED;
}

public Forward_PlayerPostThink_pre(id)
{
  if(!g_iSettingLaser[id] && g_iWasSetting[id])
    set_pev(id, pev_maxspeed, g_fPlayerSpeed[id]);
  else if(g_iSettingLaser[id] && !g_iWasSetting[id])
  {
    pev(id, pev_maxspeed, g_fPlayerSpeed[id]);
    set_pev(id, pev_maxspeed, 1.0);
  }

  g_iWasSetting[id] = g_iSettingLaser[id];
  return FMRES_IGNORED;
}

public Ham_Killed_pre(victim)
{
  remove_player_task(victim);
}

public Ham_Impulse_pre(id)
{
  if(is_user_alive(id) && !g_iSettingLaser[id] && pev(id, pev_impulse) == 100 && th_get_gamemode() == GAME_STARTED && !th_get_playerdata(id, PD_HIDDEN))
  {
    mine_start(id);
    set_pev(id, pev_impulse, 0);
    return HAM_HANDLED;
  }

  return HAM_IGNORED;
}

public mine_start(id)
{
  if(!mine_check(id) || !g_iMineCount[id])
    return PLUGIN_HANDLED;
  g_iSettingLaser[id] = true;

  message_begin(MSG_ONE_UNRELIABLE, 108, _, id);
  write_byte(1);
  write_byte(0);
  message_end();
  set_task(1.2, "mine_spawn", id);

  return PLUGIN_HANDLED;
}

public mine_spawn(id)
{
  if(!mine_check(id))
    return PLUGIN_HANDLED_MAIN;

  new ent = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"));
  if(!ent)
    return PLUGIN_HANDLED_MAIN;

  g_iMineCount[id]--;
  set_pev(ent, pev_classname, ENT_CLASS_NAME);

  engfunc(EngFunc_SetModel, ent, ENT_MODELS);

  set_pev(ent, pev_solid, SOLID_NOT);
  set_pev(ent, pev_movetype, MOVETYPE_FLY);

  set_pev(ent, pev_frame, 0);
  set_pev(ent, pev_body, 3);
  set_pev(ent, pev_sequence, TRIPMINE_WORLD);
  set_pev(ent, pev_framerate, 0);

  new Float:vOrigin[3];
  new  Float:vNewOrigin[3],Float:vNormal[3],Float:vTraceDirection[3],
    Float:vTraceEnd[3],Float:vEntAngles[3];
  pev(id, pev_origin, vOrigin);
  velocity_by_aim(id, 128, vTraceDirection);
  xs_vec_add(vTraceDirection, vOrigin, vTraceEnd);

  engfunc(EngFunc_TraceLine, vOrigin, vTraceEnd, DONT_IGNORE_MONSTERS, id, 0);
  new Float:fFraction;
  get_tr2(0, TR_flFraction, fFraction);

  if(fFraction < 1.0)
  {
    get_tr2(0, TR_vecEndPos, vTraceEnd);
    get_tr2(0, TR_vecPlaneNormal, vNormal);
  }

  xs_vec_mul_scalar(vNormal, 8.0, vNormal);
  xs_vec_add(vTraceEnd, vNormal, vNewOrigin);

  engfunc(EngFunc_SetSize, ent, Float:{-4.0, -4.0, -4.0}, Float:{4.0, 4.0, 4.0});
  engfunc(EngFunc_SetOrigin, ent, vNewOrigin);

  vector_to_angle(vNormal, vEntAngles);
  set_pev(ent, pev_angles, vEntAngles);

  // -- Calculate laser end origin.
  new Float:vBeamEnd[3], Float:vTracedBeamEnd[3];
  xs_vec_mul_scalar(vNormal, 8192.0, vNormal);
  xs_vec_add(vNewOrigin, vNormal, vBeamEnd);

  engfunc(EngFunc_TraceLine, vNewOrigin, vBeamEnd, IGNORE_MONSTERS, -1, 0);

  get_tr2(0, TR_vecPlaneNormal, vNormal);
  get_tr2(0, TR_vecEndPos, vTracedBeamEnd);

  // -- Save results to be used later.
  set_pev(ent, LASERMINE_OWNER, id);
  set_pev(ent, LASERMINE_BEAMENDPOINT, vTracedBeamEnd);
  new Float:time = get_gametime();

  set_pev(ent, LASERMINE_POWERUP, time + 2.5);
  set_pev(ent, LASERMINE_STEP, THINK_POWERUP);
  set_pev(ent, pev_nextthink, time + 0.2);

  mine_sound(ent, SOUND_POWERUP);
  remove_player_task(id);
  return 1;
}

public mine_check(id)
{
  new Float:vTraceDirection[3], Float:vTraceEnd[3],Float:vOrigin[3];

  pev(id, pev_origin, vOrigin);
  velocity_by_aim(id, 128, vTraceDirection);
  xs_vec_add(vTraceDirection, vOrigin, vTraceEnd);

  engfunc(EngFunc_TraceLine, vOrigin, vTraceEnd, DONT_IGNORE_MONSTERS, id, 0);
  new Float:fFraction,Float:vTraceNormal[3];
  get_tr2(0, TR_flFraction, fFraction);
  if(fFraction < 1.0)
  {
    get_tr2(0, TR_vecEndPos, vTraceEnd);
    get_tr2(0, TR_vecPlaneNormal, vTraceNormal);

    return true;
  }

  client_print_color(id, print_team_default, "%s %L", TH_TAG, id, "TH_SONIC_MINE");
  remove_player_task(id);
  return false;
}

player_touch_laser(mine, id)
{
  if(id < 0)
    return PLUGIN_CONTINUE;

  new owner = pev(mine, LASERMINE_OWNER);
  if(pev(mine, LASERMINE_HITING) == id)
  {

    static Float:cnt;
    static now, htime; now = floatround(get_gametime());

    pev(mine, LASERMINE_COUNT, cnt);
    htime = floatround(cnt);
    if(now - htime < 1)
      return PLUGIN_CONTINUE;
    else set_pev(mine, LASERMINE_COUNT, get_gametime());
  }
  else set_pev(mine, LASERMINE_COUNT, get_gametime());

  if(is_user_alive(id) && th_get_playerdata(id, PD_HIDDEN))
  {
    g_iLaserColor[owner] = true;
    mine_sound(id, SOUND_BEAM);
    set_pev(mine, LASERMINE_HITING, id);
  }

  return PLUGIN_CONTINUE;
}

remove_player_mines(id)
{
  new ent = 33;
  while((ent = engfunc( EngFunc_FindEntityByString, ent, "classname", ENT_CLASS_NAME)))
  {
    if(id)
    {
      if(pev(ent, LASERMINE_OWNER) != id)
        continue;

      mine_sound(ent, SOUND_STOP);
      engfunc(EngFunc_RemoveEntity, ent);
    }
    else set_pev(ent, pev_flags, FL_KILLME);
  }
}

remove_player_task(id)
{
  if(task_exists(id))
    remove_task(id);
  g_iSettingLaser[id] = false;
  return PLUGIN_CONTINUE;
}

stock mine_laser(owner, const Float:v_Origin[3], const Float:v_EndOrigin[3])
{
  new num, id, thehidden = th_get_globalinfo(GI_HIDDEN);
  static players[32];
  get_players(players, num);
  for(--num; num >= 0; num--)
  {
    id = players[num];
    if(id == thehidden) continue;
    message_begin(MSG_ONE_UNRELIABLE, SVC_TEMPENTITY, _, id);
    write_byte(TE_BEAMPOINTS);
    engfunc(EngFunc_WriteCoord,v_Origin[0]);
    engfunc(EngFunc_WriteCoord,v_Origin[1]);
    engfunc(EngFunc_WriteCoord,v_Origin[2]);
    engfunc(EngFunc_WriteCoord,v_EndOrigin[0]); //Random
    engfunc(EngFunc_WriteCoord,v_EndOrigin[1]); //Random
    engfunc(EngFunc_WriteCoord,v_EndOrigin[2]); //Random
    write_short(g_pBeamSprite);
    write_byte(0);
    write_byte(0);
    write_byte(1);  //Life
    write_byte(5);  //Width
    write_byte(0);  //wave
    write_byte(g_iColors[g_iLaserColor[owner]][0]); // r
    write_byte(g_iColors[g_iLaserColor[owner]][1]); // g
    write_byte(g_iColors[g_iLaserColor[owner]][2]); // b
    write_byte(255);
    write_byte(255);
    message_end();
  }
}

stock mine_sound(ent, sound)
{
  switch(sound)
  {
    case SOUND_POWERUP:
    {
      emit_sound(ent, CHAN_VOICE, ENT_SOUND1, VOL_NORM, ATTN_NORM, 0, PITCH_NORM);
      emit_sound(ent, CHAN_BODY , ENT_SOUND2, 0.2, ATTN_NORM, 0, PITCH_NORM);
    }
    case SOUND_ACTIVATE: emit_sound(ent, CHAN_VOICE, ENT_SOUND3, 0.5, ATTN_NORM, 1, 75);
    case SOUND_STOP:
    {
      emit_sound(ent, CHAN_BODY , ENT_SOUND2, 0.2, ATTN_NORM, SND_STOP, PITCH_NORM);
      emit_sound(ent, CHAN_VOICE, ENT_SOUND3, 0.5, ATTN_NORM, SND_STOP, 75);
    }
    case SOUND_BEAM: emit_sound(ent, CHAN_BODY, ENT_SOUND4, 1.0, ATTN_NORM, 0, PITCH_NORM);
  }
}
