#include <amxmodx>
#include <fakemeta>
#include <thehidden>
#include <amx_settings_api>

new const g_szNewRadio[][] =
{
  "thehidden/radio_behindyou.wav",
  "thehidden/radio_freshmeat.wav",
  "thehidden/radio_imcomingforu.wav",
  "thehidden/radio_imhere.wav",
  "thehidden/radio_iseeyou.wav",
  "thehidden/radio_lookup.wav",
  "thehidden/radio_overhere.wav",
  "thehidden/radio_turnaround.wav",
  "thehidden/radio_youarenext.wav"
};

new const g_szNewNames[][] =
{
  "Behind you",
  "Fresh meat",
  "I'm comming for you",
  "I'm here",
  "I see you",
  "Look up",
  "Over here",
  "Turn around",
  "You are next"
};

new const g_szDefaultSounds[] = "thehidden/hidden_death01.wav, thehidden/hidden_death02.wav, thehidden/hidden_death03.wav, \
  thehidden/hidden_death04.wav, thehidden/hidden_death05.wav, thehidden/hidden_death06.wav";
new Float:g_fNextSound[33], Array:g_aDeathSounds, g_iTheHidden;

public plugin_precache()
{
  g_aDeathSounds = ArrayCreate(TH_MAXFILELENGHT, 6);
  if(!amx_load_setting_string_arr(TH_SETTINGSFILE, "Hidden", "DEATH_SOUNDS", g_aDeathSounds))
  {
    amx_save_setting_string(TH_SETTINGSFILE, "Hidden", "DEATH_SOUNDS", g_szDefaultSounds);
    amx_load_setting_string_arr(TH_SETTINGSFILE, "Hidden", "DEATH_SOUNDS", g_aDeathSounds);
  }

  new i, sound[TH_MAXFILELENGHT];
  for(i = 0; i < ArraySize(g_aDeathSounds); i++)
  {
    ArrayGetString(g_aDeathSounds, i, sound, charsmax(sound));
    precache_sound(sound);
  }

  for(i = 0; i < sizeof(g_szNewRadio); i++)
    precache_sound(g_szNewRadio[i]);
}

public plugin_init()
{
  register_plugin("[TH] Radio commands", TH_VERSION, TH_AUTHOR);

  register_forward(FM_EmitSound, "Forward_EmitSound_pre", 0);

  register_clcmd("radio1", "radio_menu_show");
  register_clcmd("radio2", "radio_menu_show");
  register_clcmd("radio3", "radio_menu_show");
  register_clcmd("radio4", "radio_menu_show");

  new const commands[][] =
  {
    "coverme", "takepoint", "holdpos", "regroup", "followme", "takingfire",
    "go", "fallback", "sticktog", "getinpos", "stormfront", "report",
    "roger", "enemyspot", "needbackup", "sectorclear", "inposition", "reportingin", "getout", "negative", "enemydown"
  };

  for(new i = 0; i < sizeof(commands); i++)
    register_clcmd(commands[i], "cmd_blockalias");
}

public th_gamemode(mode)
{
  if(mode == GAME_STARTED)
    g_iTheHidden = th_get_globalinfo(GI_HIDDEN);
}

public Forward_EmitSound_pre(id, channel, sample[])
{
  if(!is_user_connected(id) || g_iTheHidden != id)
    return FMRES_IGNORED;

  if(equal(sample, "player/die", 10) || equal(sample, "player/death6.wav"))
  {
    static sound[TH_MAXFILELENGHT];
    ArrayGetString(g_aDeathSounds, random(ArraySize(g_aDeathSounds)), sound, charsmax(sound));
    emit_sound(id, CHAN_AUTO, sound, 1.0, ATTN_NORM, 0, PITCH_NORM);
    g_iTheHidden = false;
    return FMRES_SUPERCEDE;
  }

  return FMRES_IGNORED;
}

public cmd_blockalias(id)
  return PLUGIN_HANDLED;

public radio_menu_show(id)
{
  if(is_user_alive(id) && th_get_gamemode() == GAME_STARTED && th_get_playerdata(id, PD_HIDDEN))
  {
    static menu;
    menu = menu_create("Radio menu", "radio_menu_handle");

    new data[3];
    for(new i = 0; i < sizeof(g_szNewRadio); i++)
    {
      num_to_str(i, data, charsmax(data));
      menu_additem(menu, g_szNewNames[i], data, 0);
    }

    menu_setprop(menu, MPROP_EXIT, MEXIT_NEVER); 
    menu_setprop(menu, MPROP_PERPAGE, 0); 
    menu_display(id, menu);
  }

  return PLUGIN_HANDLED;
}

public radio_menu_handle(id, menu, item)
{
  if(!is_user_alive(id) || th_get_gamemode() != GAME_STARTED || g_fNextSound[id] > get_gametime() || !th_get_playerdata(id, PD_HIDDEN))
  {
    menu_destroy(menu);
    return PLUGIN_HANDLED;
  }

  new access, callback, num[3];
  menu_item_getinfo(menu, item, access, num, charsmax(num), _, _, callback);
  menu_destroy(menu);

  new pick = str_to_num(num);
  g_fNextSound[id] = GetWavDuration(g_szNewRadio[pick]);
  if(g_fNextSound[id] < 1.5)
    g_fNextSound[id] = get_gametime() + 1.5;
  else g_fNextSound[id] = get_gametime() + g_fNextSound[id];

  emit_sound(id, CHAN_AUTO, g_szNewRadio[pick], 1.0, ATTN_NORM, 0, PITCH_NORM);

  return PLUGIN_HANDLED;
}

// By Arkshine
Float:GetWavDuration(const WavFile[])
{
  new Frequence[4];
  new Bitrate[2];
  new DataLength[4];
  new File;
  static fulldest[64];

  // --| Open the file.
  formatex(fulldest, charsmax(fulldest), "sound/%s", WavFile);
  File = fopen(fulldest, "rb");

  // --| Get the frequence from offset 24. ( Read 4 bytes )
  fseek(File, 24, SEEK_SET);
  fread_blocks(File, Frequence, 4, BLOCK_INT);
  
  // --| Get the bitrate from offset 34. ( read 2 bytes )
  fseek(File, 34, SEEK_SET); 
  fread_blocks(File, Bitrate, 2, BLOCK_BYTE);
  
  // --| Search 'data'. If the 'd' not on the offset 40, we search it.
  if(fgetc(File) != 'd') while(fgetc(File) != 'd' && !feof(File)) {}
  
  // --| Get the data length from offset 44. ( after 'data', read 4 bytes )
  fseek(File, 3, SEEK_CUR); 
  fread_blocks(File, DataLength, 4, BLOCK_INT);

  // --| Close file.
  fclose(File);
  
  // --| Calculate the time. ( Data length / ( frequence * bitrate ) / 8 ).
  return float(DataLength[0]) / (float(Frequence[0] * Bitrate[0]) / 8.0);
}
